#!/usr/bin/perl -w
#########################################################################
#
# File:         check_traffic.pl
# Description:  Nagios check plugins to check network interface traffic with perl.
#		A clone from check_traffic.sh and some Code from check_snmp_int.pl.
# Language:     Perl/5.8
# Version:	0.0.4
# Date:		2009-02-20
# Corp.:	Chenlei
# Author:	chnl@163.com (U can msn me with this.)
# WWW:		http://bbs.itnms.net
#########################################################################
# Bugs:
# The Latest Version will be released in http://bbs.itnms.net.
# You can send bugs to http://bbs.itnms.net,
# or email to me directly: chnl@163.com
#########################################################################
# Todo:
#
#########################################################################
# ChangeLog:
#
# Version:	0.0.4
# Date:		2009-02-20
# Fix the bug "Use of uninitialized value in concatenation (.) or string at check_traffic.pl line" when getting the hist data.
# When write or read file error(and some other condition), use Unknow instead of Warning severity.
# If get a negative time interval value here, exit with unknow.
#
# Version 0.0.3
# 2009-01-14
# If get a negative value here, exit with unknow.
#
# Version 0.0.2
# 2009-01-06
# Add the code get the OS type.
#
# Version 0.0.1
# 2008-12-05
# Original Version
#########################################################################
# Heh, just a ad here :), for my honey.
# http://shop35165045.taobao.com/
############################
#
# Exit values:
# ------------
#    0		OK
#    1		Warning
#    2		Cirital
#    3		Unknown
#    Others	Unknown
#
# ----------------------------------------------------------------------
# These are Parameters from external
# -h 
#	Get the help message
#
# -v 
#	Verbose mode, to debug some messages out to the console.
#
# -V    1|2c|3
#	Specify the version of snmp
#
# -C    Community
#	Specify the Community
#
# -H    host
#	Specify the host
#
# -6    Use 64 bit counter, ifHC*  instead of if*.
#
# -r    Use Range instead of single value in warning and critical Threshold;
#
# -I    interface
#	Specify the interface
#
# -L    List all Interfaces on specify host
#
# -B/b  Switch to B/s or bps, default is -b, bps
#
# -K/M  Switch to K or M (bsp,B/s), default is -K 
#
# -w    Warning value Kbps, in and out
# -c    Critical value Kbps, in and out
# 	Set Warning and Critical Traffic value

use strict;
use Net::SNMP;
use Getopt::Long;

my $Version='0.0.1/2008-12-05';
my $MinInterval=30;
my $TIMEOUT = 10;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my %status=(1=>'UP',2=>'DOWN',3=>'TESTING',4=>'UNKNOWN',5=>'DORMANT',6=>'NotPresent',7=>'lowerLayerDown');

my $descr_table = '1.3.6.1.2.1.2.2.1.2';
my $oper_table = '1.3.6.1.2.1.2.2.1.8.';
my $ii_table = '1.3.6.1.2.1.2.2.1.10.';
my $ii_table_64 = '1.3.6.1.2.1.31.1.1.1.6.';
my $io_table = '1.3.6.1.2.1.2.2.1.16.';
my $io_table_64 = '1.3.6.1.2.1.31.1.1.1.10.';

my $o_help=		undef;	
my $o_verbose=		undef;	
my $o_64bitcounter=	undef;	
my $o_range = 		undef; 	
my $o_version=		undef;	
my $o_pversion=		undef;	
my $o_community = 	undef; 	
my $o_hostname = 	undef; 	
my $o_port = 		161; 	
my $o_list = 		undef; 	
my $o_index = 		undef; 	
my $o_warning =		undef; 	
my $o_critical =	undef; 	
my $o_K =		undef; 	
my $o_M =		undef; 	
my $o_B =		undef; 	
my $o_b =		undef; 	

sub read_file { 
	my $msg=undef;
	my $file=shift;
	if (! open CHD, "$file"){
		print "$0: Can't read data from $file: $!";
		return 0;
	}else{
		$msg = <CHD>;
		if (! defined($msg)) {
			$msg="null";
		}
		verb("Hist Data from file: $msg");
		return $msg;
	}
}

sub write_file { 
	my $msg=shift;
	my $file=shift;
	if (! open CHD, ">$file"){
		print "$0: Can't write data to $file: $!";
		return 0;
	}else{
		verb("Data to file: $msg");
		print CHD "$msg\n";
		return 1;
	}
}

sub p_version {
	print "check_traffic.pl version : $Version\n"; 
}

sub print_usage {
	print <<EOT;
Usage:
$0 [ -v ] [ -6 ] [ -r ] -V 1|2c|3 -C snmp-community -H host [ -L ] -I interface -w in,out-warning-value  -c in,out-critical-value -K/M -B/b 
Example:
$0 -V 2c -C public -H 127.0.0.1 -I 4 -w 200,100 -c 300,200 -K -B
Or -r to use Range Value Options:
$0 -V 2c -C public -H 127.0.0.1 -I 4 -r -w 200-300,100-200 -c 100-400,50-250 -K -B
If you don't use -K/M -B/b options, default -K -b, corresponding to Kbps.
Make sure that the check interval greater than 5 Seconds.
Or modify the Min_Interval var in this file Line 180.
And, if you want in Verbose mode, use -v, to check the debug messages at console.
Or use $0 [ -v ] -V 1|2c|3 -C snmp-community -H host -L 
To list all interfaces on specify host.
EOT
}

sub isnnum {
	my $num = shift;
	if ( $num =~ /^(\d+\.?\d*)|(^\.\d+)$/ ) { 
		return 0 ;
	}
	return 1;
}

sub help {
	print "\nCheck Traffic for Nagios version ",$Version,"\n";
	print "by chnl\@163.com(Email or MSN)\n\n";
	print_usage();
	print <<EOT;
############
Some extra help message here.
############
EOT
}

sub verb {
	my $t=shift;
	if (defined($t)) {
		print $t,"\n" if defined($o_verbose);
	}
}

sub get_session {
	my $host=shift;
	my $port=shift;
	my $version=shift;
	my $community=shift;
	my $session = undef;
	my $error = undef;
	verb("$host $port $version $community");
	($session, $error) = Net::SNMP->session(
		-hostname  => $host,
		-version   => $version,
		-community => $community,
		-port      => $port,
		-timeout   => $TIMEOUT
	);
	if (!defined($session)) {
		printf("ERROR opening session: %s.\n", $error);
		exit $ERRORS{"UNKNOWN"};
	}else{
		return $session;
	}
}

sub list_if {
	my $session=get_session(@_);
	my $resultdt = $session->get_table( 
		Baseoid => $descr_table 
	);
	if (!defined($resultdt)) {
		printf("ERROR: %s.\n", $session->error);
		$session->close;
		exit $ERRORS{"UNKNOWN"};
	}
	foreach my $key ( sort keys %$resultdt) {
		verb("$key");
		my @mib_array=split(/\./, $key);
		verb("@mib_array");
		my $index_num=pop(@mib_array);
		verb("$index_num");
		print("Index : $index_num, Desc : $$resultdt{$key}\n");
	}
	$session->close;
	exit $ERRORS{"UNKNOWN"};
}

sub check_options {
	Getopt::Long::Configure ("bundling");
	GetOptions(
		'v'	=> \$o_verbose,		'verbose'	=> \$o_verbose,
		'h'     => \$o_help,    	'help'        	=> \$o_help,
		'6'	=> \$o_64bitcounter,	'64'		=> \$o_64bitcounter,
		'r'	=> \$o_range,		'range'		=> \$o_range,
		'V:s'	=> \$o_version,		'version'	=> \$o_version,
		'P:s'	=> \$o_pversion,		'pversion'	=> \$o_pversion,
		'C:s'   => \$o_community,	'community:s'	=> \$o_community,
		'H:s'   => \$o_hostname,	'hostname:s'	=> \$o_hostname,
		'p:i'   => \$o_port,   		'port:i'	=> \$o_port,
		'L'	=> \$o_list,		'list'		=> \$o_list,
		'I:i'	=> \$o_index,		'index'		=> \$o_index,
		'w:s'	=> \$o_warning,		'warning'	=> \$o_warning,
		'c:s'	=> \$o_critical,	'critical'	=> \$o_critical,
		'K'	=> \$o_K,		'kilo'		=> \$o_K,
		'M'	=> \$o_M,		'mega'		=> \$o_M,
		'B'	=> \$o_B,		'Byte'		=> \$o_B,
		'b'	=> \$o_b,		'bit'		=> \$o_b
	);
	if (defined ($o_help) ) { 
		help(); 
		exit $ERRORS{"UNKNOWN"};
	}
	if (defined ($o_pversion) ) { 
		p_version(); 
		exit $ERRORS{"UNKNOWN"};
	}
	if ( ! defined($o_version) || ! defined($o_community) || ! defined($o_hostname) ) {
		print_usage(); 
		exit $ERRORS{"UNKNOWN"};
	}
	if (defined($o_64bitcounter)){
		if (eval "require bigint") {
			use bigint;
		}else{ 
			print "Need bigint module for 64 bit counters\n"; 
			exit $ERRORS{"UNKNOWN"};
		}
	       	if (($o_version eq "1") ){
			print "Use 64Bit counter, must use snmp v2c or later!\n";
			exit $ERRORS{"UNKNOWN"};
		}
	}
	if (defined($o_list)){
		if (defined($o_index)) {
			print "Can't mix --list and --index option!\n";
			print_usage();
			exit $ERRORS{"UNKNOWN"};
		}
		list_if($o_hostname, $o_port, $o_version, $o_community);
	}
	if ( ! defined($o_warning) || ! defined($o_critical) ) {
		print_usage(); 
		exit $ERRORS{"UNKNOWN"};
	}
	if ( defined($o_K) && defined($o_M) ) {
		print "Can't mix -K and -M option!\n";
		exit $ERRORS{"UNKNOWN"};
	}
	if ( defined($o_B) && defined($o_b) ) {
		print "Can't mix -B and -b option!\n";
		exit $ERRORS{"UNKNOWN"};
	}
	if (isnnum($o_index)) {
		print "Index must be a number!\n";
		print "$o_index\n";
		exit $ERRORS{"UNKNOWN"};
	}
}
check_options() ;

my @wva=split(/\,/, $o_warning);
my @cva=split(/\,/, $o_critical);
verb("@wva @cva");
my $wvl=@wva;
my $cvl=@cva;
verb("$wvl $cvl");
if (($wvl != 2) || ($cvl !=2)){
	print "Warning/Critical value set error.\n";
	print_usage(); 
	exit $ERRORS{"UNKNOWN"};
}
my $wvi;
my $wvo;
my $cvi;
my $cvo;
my $wvis;
my $wvie;
my $wvos;
my $wvoe;
my $cvis;
my $cvie;
my $cvos;
my $cvoe;
if ( $o_range){
	verb("In range value parse process");
	my @wvia=split(/\-/, $wvi);
	my @wvoa=split(/\-/, $wvo);
	my @cvia=split(/\-/, $cvi);
	my @cvoa=split(/\-/, $cvo);
	$wvis = $wvia[0];
	$wvie = $wvia[1];
	$wvos = $wvoa[0];
	$wvoe = $wvoa[1];
	$cvis = $cvia[0];
	$cvie = $cvia[1];
	$cvos = $cvoa[0];
	$cvoe = $cvoa[1];
	if ( ($wvis >= $wvie) || ($wvos >= $wvoe) || ($cvis >= $cvie) || ($cvos >= $cvoe) || ($wvis <= $cvis) || ($wvie >= $cvie) || ($wvos <= $cvos) || ($wvoe >= $cvoe) ) {
		verb("Range value are $wvis $wvie $wvos $wvoe $cvis $cvie $cvos $cvoe");
		verb("--range warning/critical value check failed");
		print "--range warning/critical value set error.\n";
		print_usage();
		exit $ERRORS{"UNKNOWN"};
	}
}else{
		$wvi = $wva[0];
		$wvo = $wva[1];
		$cvi = $cva[0];
		$cvo = $cva[1];
		verb("$wvi $wvo $cvi $cvo");
		if (( $wvi >=  $cvi ) ||  ($wvo >= $cvo)){
			print "Critical Value must  greater than Warning Value.\n";
			exit $ERRORS{"UNKNOWN"};
		}
}

my $ct_hist_file;
if ( $^O =~ /MSWin32/ ){
	$ct_hist_file="check_traffic_" . $o_hostname . "_" . $o_index . ".hist_dat";
}else{
	$ct_hist_file="/var/tmp/check_traffic_" . $o_hostname . "_" . $o_index . ".hist_dat";
}
my $output;
my $tips;
my $severity;
my $isFirstTime="False";

verb("Hist data file is $ct_hist_file");
verb("is First Time is $isFirstTime");

if ( ! -e $ct_hist_file) {
	if (! write_file("it's,first,time","$ct_hist_file")){
		exit $ERRORS{"UNKNOWN"};
	}
	$output="It's the first time for this plugins run(or the old hist_data file had been revmoved). We'll get the data from the next time.";
	$severity=$ERRORS{"UNKNOWN"};
	$tips="Unknow";
	$isFirstTime="True";
}
my $hist_data="null";
my $hist_dt="null";
my $hist_iic="null";
my $hist_ioc="null";

if ($isFirstTime eq "False"){
	$hist_data=read_file("$ct_hist_file");
	if (! $hist_data){
		exit $ERRORS{"UNKNOWN"};
	}
	verb("Hist data from file: $hist_data");

	my @hist_a = split(/,/,$hist_data);
	$hist_dt=$hist_a[0];
	$hist_iic=$hist_a[1];
	$hist_ioc=$hist_a[2];

	if (! defined($hist_dt)) {
		$hist_dt="null";
	}
	if (! defined($hist_iic)) {
		$hist_iic="null";
	}
	if (! defined($hist_ioc)) {
		$hist_ioc="null";
	}

	verb("Hist data, dt/iic/ioc: $hist_dt,$hist_iic,$hist_ioc");
	if((isnnum($hist_dt)) || (isnnum($hist_iic)) || (isnnum($hist_ioc))){
		$output="It's the first time for this plugins run(or the old hist_data file had been revmoved). We'll get the data from the next time.";
		$severity=$ERRORS{"UNKNOWN"};
		$tips="Unknow";
		$isFirstTime="True";
	}
}

my $dt =time();
my $interval;
if ($isFirstTime eq "False"){
$interval = $dt - $hist_dt;
	if ( $interval < $MinInterval ) {
		print "The check interval must greater than $MinInterval, but now it's $interval.\n";
		exit $ERRORS{"UNKNOWN"};
	}
}
my $session = get_session($o_hostname, $o_port, $o_version, $o_community);

my $op_oid=$oper_table. $o_index;
my $resultos = $session->get_request( 
	-varbindlist => [$op_oid]
);
if (!defined($resultos)){
	printf("ERROR: %s.\n", $session->error);
	$session->close;
	exit $ERRORS{"UNKNOWN"};
}
if( $resultos->{$op_oid} != 1 ) {
	printf("Interface $o_index status is status %s.", $status{$resultos->{$op_oid}});
	$session->close;
	exit $ERRORS{"CRITICAL"};
}
my $ii_oid;
my $io_oid;
my $id_oid = $descr_table . "." .  $o_index;

if (defined($o_64bitcounter)){
	$ii_oid = $ii_table_64 . $o_index;
	$io_oid = $io_table_64 . $o_index;
}
else {
	$ii_oid = $ii_table . $o_index;
	$io_oid = $io_table . $o_index;
}

my $resultii = $session->get_request( 
	-varbindlist => [$ii_oid]
);
if (!defined($resultii)){
	printf("ERROR: %s.\n", $session->error);
	$session->close;
	exit $ERRORS{"UNKOWN"};
}

my $resultio = $session->get_request( 
	-varbindlist => [$io_oid]
);
if (!defined($resultio)){
	printf("ERROR: %s.\n", $session->error);
	$session->close;
	exit $ERRORS{"UNKOWN"};
}

my $resultid = $session->get_request( 
	-varbindlist => [$id_oid]
);
if (!defined($resultid)){
	printf("ERROR: %s.\n", $session->error);
	$session->close;
	exit $ERRORS{"UNKOWN"};
}
my $iic = $resultii->{$ii_oid};
my $ioc = $resultio->{$io_oid};
my $id = $resultid->{$id_oid};

if (isnnum($iic) || isnnum($ioc)){
	printf("ERROR: iic/ioc value is  %s.\n", "$iic" . " " . "$ioc");
	$session->close;
	exit $ERRORS{"UNKNOWN"};
}
$session->close;

verb("dt: $dt");
verb("iic: $iic");
verb("ioc: $ioc");

verb("$dt,$iic,$ioc");
if (! write_file("$dt,$iic,$ioc","$ct_hist_file")){
	print "$0: Can't write to $ct_hist_file: $!";
	$session->close;
	exit $ERRORS{"UNKNOWN"};
}


if( $isFirstTime eq "False"){
	verb("Interval is $interval");
	my $diffIn = $iic - $hist_iic;
	verb("DiffIn is $diffIn");
	my $diffOut = $ioc - $hist_ioc;
	verb("DiffOut is $diffOut");
	my $in = $diffIn / $interval;
	verb("Origin in is $in");
	my $out = $diffOut / $interval;
	verb("Origin out is $out");
	if ($interval < 0){
		print "We got a negative time interval value here.\n";
		exit $ERRORS{"UNKNOWN"};
	}
	if (($diffIn < 0) || ($diffOut <0 )){
		print "Maybe 32 bit counter overflow, because we got a negative value here.\n";
		exit $ERRORS{"UNKNOWN"};
	}
	my $unit_1="";
	my $unit_2="B/s";
	verb("Origin unit_1/unit_2 are $unit_1 $unit_2");

	if (defined($o_K)){
		$in =  $in / 1024 ;
		$out =  $out /1024 ;
		$unit_1 = "K"
	}
	if (defined($o_b)){
		$in =  $in * 8 ;
		$out =  $out * 8 ;
		$unit_2 = "bps";
	}
	if (defined($o_M)){
		$in =  $in / 1024 /1024 ;
		$out =  $out /1024 /1024 ;
		$unit_1 = "M"
	}
		
	verb("Final in is $in");
	verb("Final out is $out");
	verb("Final unit_1/unit_2 are $unit_1 $unit_2");
	my $total = $in + $out;
	verb("Total is $total");
	if ($o_range){
		if ( ($in >= $wvis) && ($in < $wvie) && ($out >= $wvos) && ($out < $wvoe) ) {
			$severity=$ERRORS{"OK"};
			$tips="OK";
		}
		elsif( ($in < $cvis) || ($in >= $cvie) || ($out < $cvos) || ($out >= $cvoe)){
			$severity=$ERRORS{"CRITICAL"};
			$tips="Critical";
		}
		else{
			$severity=$ERRORS{"WARNING"};
			$tips="Warning";
		}
		$wvi=0;
		$wvo=0;
		$cvi=0;
		$cvo=0;
	}else{
		if ( ($in < $wvi) && ($out < $wvo)){
			$severity=$ERRORS{"OK"};
			$tips="OK";
		}elsif ( ($in >= $cvi) || ($out >= $cvo )){
			$severity=$ERRORS{"CRITICAL"};
			$tips="Critical";
		}else{
			$severity=$ERRORS{"WARNING"};
			$tips="Warning";
		}
	}

	my $wv = $wvi + $wvo;
	my $cv = $cvi + $cvo;
	$output="The Traffic In is $in$unit_1$unit_2, Out is $out$unit_1$unit_2, Total is $total$unit_1$unit_2. The Check Interval is $interval". "s\|In\=$in$unit_1$unit_2\;$wvi\;$cvi\;0\;0 Out\=$out$unit_1$unit_2\;$wvo\;$cvo\;0\;0 Total\=$total$unit_1$unit_2\;$wv\;$cv\;0\;0 Interval\=$interval" . "s\;1200\;1800\;0\;0\n";
}
print "$tips" . " - "  . "$output";
exit $severity;

#End of check_traffi.pl
